import React from 'react'
import PropTypes from 'prop-types'
import Dropdown from './Dropdown'
import Button from './Button'
import Loader from './Loader'
import { withStyles } from '../helpers/withStyles'
import { withStylesPropTypes } from 'react-with-styles'

const propTypes = {
    ...withStylesPropTypes,
    mainOption: PropTypes.object,
    disabled: PropTypes.bool,
    loading: PropTypes.bool,
    options: PropTypes.arrayOf(PropTypes.object).isRequired,
    children: PropTypes.any,
    onClick: PropTypes.func,
    fullWidth: PropTypes.bool,
    block: PropTypes.bool,
    compact: PropTypes.bool,
    inverse: PropTypes.bool,
    mono: PropTypes.bool,
    noSpacing: PropTypes.bool,
    shimmer: PropTypes.bool,
    soft: PropTypes.bool,
    primary: PropTypes.bool,
    rectangular: PropTypes.bool,
    secondary: PropTypes.bool,
    short: PropTypes.bool,
    transparent: PropTypes.bool,
    closeOnSelect: PropTypes.bool
};

const defaultProps = {
    mainOption: { label: '', value: null },
    options: []
};

const ComboButton = (
    {
        styles,
        css,
        theme,

        mainOption,
        disabled,
        loading,
        options,
        className,
        children,
        onClick,
        fullWidth,
        block,
        compact,
        inverse,
        mono,
        noSpacing,
        shimmer,
        soft,
        primary,
        rectangular,
        secondary,
        transparent,
        closeOnSelect,

        ...props
    }
) => {
    return (
        <div
            {...props}
            {...css(
                styles.comboButton,
                loading && styles.comboButton_loading,
                disabled && styles.comboButton_disabled,
                fullWidth && styles.comboButton_fullWidth,
                block && styles.comboButton_block,
                inverse && styles.comboButton_inverse,
                mono && styles.comboButton_mono,
                noSpacing && styles.comboButton_noSpacing,
                shimmer && styles.comboButton_soft,
                soft && styles.comboButton_soft,
                primary && styles.comboButton_primary,
                rectangular && styles.comboButton_rectangular,
                secondary && styles.comboButton_secondary,
                transparent && styles.comboButton_transparent,
            )}
        >
            <div
                {...css(
                    styles.comboButton__content,
                    loading && styles.comboButton__content_loading,
                    shimmer && styles.comboButton__content_shimmer,
                    rectangular && styles.comboButton__content_rectangular
                )}
            >
                <Button
                    noSpacing
                    short
                    fullWidth={block || fullWidth }
                    compact={compact}
                    mono={mono}
                    soft={soft}
                    primary={primary}
                    rectangular={rectangular}
                    secondary={secondary}
                    transparent={transparent}
                    inverse={inverse}
                    noBorder={mono || secondary || inverse}
                    onClick={() => onClick && onClick(mainOption.value)}
                >
                    {mainOption.label}
                </Button>
                <div
                    {...css(
                        styles.comboButton__separator,
                        (inverse || transparent || soft) && styles.comboButton__separator_dark,
                        mono && styles.comboButton__separator_mono,
                        secondary && styles.comboButton__separator_secondary,
                    )}
                />
                <Dropdown
                    combo
                    button
                    noSpacing
                    short
                    compact
                    mono={mono}
                    soft={soft}
                    primary={primary}
                    rectangular={rectangular}
                    secondary={secondary}
                    transparent={transparent}
                    inverse={inverse}
                    noBorder={mono || secondary || inverse}
                    onSelectOption={option => onClick && onClick(option.value)}
                    options={options}
                    closeOnSelect={closeOnSelect}
                >
                    { children }
                </Dropdown>
            </div>
            {
                loading &&
                <div {
                    ...css(
                        styles.comboButton__loader,
                        shimmer && styles.comboButton__loader_shimmer,
                        rectangular && styles.comboButton__content_rectangular
                    )}>
                    { !shimmer && <Loader small primary={inverse && soft} secondary={secondary} mono={mono} /> }
                </div>
            }
        </div>
    )
};

ComboButton.propTypes = propTypes;
ComboButton.defaultProps = defaultProps;

const ComboButtonWithStyles = withStyles(({ color, units, animation, radius }) => ({
    comboButton: {
        height: units(2.5),
        borderRadius: radius,
        display: 'inline-block',
        position: 'relative',
        backgroundColor: color.secondary,
        paddingTop: units(0.25),
        paddingRight: units(0.125),
        paddingBottom: units(0.25),
        paddingLeft: units(0.125),
        marginTop: 0,
        marginRight: units(0.125),
        marginBottom: units(0.5),
        marginLeft: units(0.125),
        boxSizing: 'border-box',
    },

    comboButton_loading: {
        opacity: 1
    },

    comboButton__content: {
        display: 'flex',
        alignItems: 'center',
        overflow: 'hidden',
        borderRadius: radius,
    },

    comboButton__content_shimmer: {
        minWidth: units(6)
    },

    comboButton__content_loading: {
        opacity: 0
    },

    comboButton__content_rectangular: {
        borderRadius: 0
    },

    comboButton__separator: {
        width: 1,
        height: units(1.5),
        marginRight: units(0.125),
        marginLeft: units(0.125),
        backgroundColor: color.fogStrong
    },

    comboButton__separator_dark: {
        backgroundColor: color.shadow
    },

    comboButton__separator_mono: {
        backgroundColor: color.black
    },

    comboButton__separator_secondary: {
        backgroundColor: color.secondary
    },

    comboButton__loader: {
        position: 'absolute',
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
        borderRadius: radius,
    },

    comboButton__loader_shimmer: {
        backgroundColor: color.shimmer.from,
        animationDirection: 'alternate',
        animationDuration: '1s',
        animationFillMode: 'forwards',
        animationIterationCount: 'infinite',
        animationName: animation.shimmer,
        animationTimingFunction: 'ease-in-out'
    },

    comboButton_disabled: {
        opacity: .5,
        pointerEvents: 'none'
    },

    comboButton_soft: {
        backgroundColor: color.blueGrayLighter,
    },

    comboButton_fullWidth: {
        width: '100%'
    },

    comboButton_block: {
        width: '100%',
        textAlign: 'left',
        marginRight: 0,
        marginLeft: 0,
    },

    comboButton_inverse: {
        backgroundColor: color.white,
    },

    comboButton_mono: {
        backgroundColor: color.transparent,
        borderColor: color.black,
        borderWidth: 1,
        borderStyle: 'solid',
    },

    comboButton_noSpacing: {
        marginBottom: 0
    },

    comboButton_primary: {
        backgroundColor: color.primary,
    },

    comboButton_secondary: {
        backgroundColor: color.transparent,
        borderColor: color.secondary,
        borderWidth: 1,
        borderStyle: 'solid',
    },

    comboButton_transparent: {
        backgroundColor: color.transparent,
    },

    comboButton_rectangular: {
        borderRadius: 0
    },

}))(ComboButton);

export default ComboButtonWithStyles
