import React, { Component } from 'react'
import OutsideClickHandler from 'react-outside-click-handler'
import Transition from 'react-transition-group/Transition'
import Button from './Button'
import Icon from './Icon'
import { css, withStyles } from '../helpers/withStyles'
import { withStylesPropTypes } from 'react-with-styles'
import PropTypes from 'prop-types'

const dropdownPropTypes = {
    ...withStylesPropTypes,
    disabled: PropTypes.bool,
    loading: PropTypes.bool,
    fullWidth: PropTypes.bool,
    block: PropTypes.bool,
    combo: PropTypes.bool,
    compact: PropTypes.bool,
    icon: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
    inverse: PropTypes.bool,
    mono: PropTypes.bool,
    noBorder: PropTypes.bool,
    noSpacing: PropTypes.bool,
    shade: PropTypes.bool,
    soft: PropTypes.bool,
    primary: PropTypes.bool,
    rectangular: PropTypes.bool,
    secondary: PropTypes.bool,
    short: PropTypes.bool,
    transparent: PropTypes.bool,
    children: PropTypes.node,
    button: PropTypes.bool,
    label: PropTypes.string,
    only: PropTypes.bool,
    relative: PropTypes.bool,
    left: PropTypes.bool,
    options: PropTypes.arrayOf(PropTypes.object),
    onSelectOption: PropTypes.func,
    closeOnSelect: PropTypes.bool
};

const dropdownMenuPropTypes = {
    ...withStylesPropTypes,
    children: PropTypes.any.isRequired,
    left: PropTypes.bool,
    isOpen: PropTypes.bool,
    insideAButton: PropTypes.bool
};

const dropdownDefaultProps = {
    options: []
}

class Dropdown extends Component {
    constructor(props) {
        super(props);
        this.state = {
            isOpen: false,
            closedByOutsideClick: false
        }
    }

    toggleMenu() {
        this.setState(({ isOpen, closedByOutsideClick }) => (
            closedByOutsideClick && !isOpen ? { closedByOutsideClick: false} : { isOpen: !isOpen }
        ))
    }

    onClickOutsideMenu() {
        this.setState(state => ({ isOpen: false, closedByOutsideClick: state.isOpen }))
    }

    onSelectOption(option) {
        this.props.onSelectOption && this.props.onSelectOption(option);
        this.props.closeOnSelect && this.toggleMenu();

    }

    render() {
        const {
            styles,
            button,
            label,
            children,
            only,
            relative,
            left,
            options,
            disabled,
            loading,
            fullWidth,
            block,
            combo,
            compact,
            icon,
            inverse,
            mono,
            noBorder,
            noSpacing,
            shade,
            soft,
            shimmer,
            primary,
            rectangular,
            secondary,
            short,
            transparent,
        } = this.props;

        return (
            <div
                {...css(
                    styles.dropdown__container,
                    button && styles.dropdown__container_asAButton,
                    noSpacing && styles.dropdown__container_noSpacing,
                    !combo && (button || only || relative) && styles.dropdown__container_relative,
                )}
            >
                { button ?
                    <Button
                        noSpacing
                        disabled={disabled}
                        loading={loading}
                        shimmer={shimmer}
                        fullWidth={fullWidth}
                        block={block}
                        compact={compact}
                        inverse={inverse}
                        mono={mono}
                        noBorder={noBorder}
                        soft={soft}
                        primary={primary}
                        rectangular={rectangular}
                        secondary={secondary}
                        shade={shade}
                        short={short}
                        transparent={transparent}
                        type="button"

                        onClick={() => this.toggleMenu()}
                        {...css(
                            styles.dropdown,
                            styles.dropdown_asAButton
                        )}
                    >
                        { label }
                        {
                            icon ?
                                typeof label === 'string'
                                    ? <Icon name={icon} primary smaller noPadding />
                                    : icon
                            : <svg
                                {...css(
                                    styles.dropdown__arrow,
                                    label && styles.dropdown__arrow_leftMarged,
                                    mono && styles.dropdown__arrow_mono,
                                    (soft || inverse || transparent) && styles.dropdown__arrow_primary,
                                    secondary && styles.dropdown__arrow_secondary,
                                )}
                                xmlns="http://www.w3.org/2000/svg"
                                viewBox="0 0 10.4 6.3"
                            >
                                <path d="M5.2,6.3A.91.91,0,0,1,4.5,6L.3,1.8A1,1,0,0,1,.3.4,1,1,0,0,1,1.7.4L5.2,3.9,8.7.3a1,1,0,0,1,1.4,0,1,1,0,0,1,0,1.4L5.9,6a4.88,4.88,0,0,1-.7.3Z"/>
                            </svg>
                        }
                    </Button>
                :
                    <button
                        type="button"
                        onClick={() => this.toggleMenu()}
                        {...css(
                            styles.dropdown,
                            only && styles.dropdown_only,
                        )}
                    >
                        <svg
                            {...css(
                                styles.dropdown__arrow,
                                mono && styles.dropdown__arrow_mono,
                                (soft || inverse || transparent || primary) && styles.dropdown__arrow_primary,
                                secondary && styles.dropdown__arrow_secondary
                            )}
                            xmlns="http://www.w3.org/2000/svg"
                            viewBox="0 0 10.4 6.3"
                        >
                            <path d="M5.2,6.3A.91.91,0,0,1,4.5,6L.3,1.8A1,1,0,0,1,.3.4,1,1,0,0,1,1.7.4L5.2,3.9,8.7.3a1,1,0,0,1,1.4,0,1,1,0,0,1,0,1.4L5.9,6a4.88,4.88,0,0,1-.7.3Z"/>
                        </svg>
                    </button>
                }
                <DropdownMenuWithStyles
                    isOpen={this.state.isOpen}
                    onClickOutside={() => this.onClickOutsideMenu()}
                    left={left}
                    insideAButton={button}
                    combo={combo}
                >
                    { children ? children :
                        <ul
                            {...css(
                                styles.dropdown__list
                            )}
                        >
                            { options.map(option => {
                                return (
                                    <li key={option.value} {...css(styles.dropdown__list__item)}>
                                        <Button short compact inverse noSpacing block
                                            onClick={() => this.onSelectOption(option)}
                                        >
                                            {option.label}
                                        </Button>
                                    </li>
                                )
                            }) }
                        </ul>
                    }
                </DropdownMenuWithStyles>
            </div>
        )
    }
}

class DropdownMenu extends Component {
    render() {
        const { children, combo, left, isOpen, insideAButton, styles, onClickOutside } = this.props;

        return (
            <OutsideClickHandler onOutsideClick={ () => onClickOutside() }>
                <Transition
                    in={isOpen}
                    timeout={300}
                    mountOnEnter
                    unmountOnExit
                    onEnter={node => node.scrollTop}
                >
                    {(state) => (
                        <div
                            {...css(
                                styles.dropdownMenu,
                                !combo && insideAButton && styles.dropdownMenu_insideAButton,
                                left ? styles.dropdownMenu_left : styles.dropdownMenu_right,
                                (state === 'entering' || state === 'entered') && styles.dropdownMenu_entering,
                                (state === 'exiting' || state === 'exited') && styles.dropdownMenu_exiting,
                            )}
                        >
                            { children }
                        </div>
                    )}
                </Transition>
            </OutsideClickHandler>
        )
    }
}

Dropdown.propTypes = dropdownPropTypes;
Dropdown.defaultProps = dropdownDefaultProps;
DropdownMenu.propTypes = dropdownMenuPropTypes;

const DropDownWithStyles = withStyles(({ color, units }) => ({
    dropdown: {
        minWidth: units(1.5),
        minHeight: units(1.5),
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
        outline: 'none',
        cursor: 'pointer',
        borderWidth: 0,
        background: color.transparent
    },

    dropdown_asAButton: {
        margin: 0
    },

    dropdown_only: {
        minWidth: units(2.5),
        height: units(2.5)
    },

    dropdown__arrow: {
        width: units(.5),
        fill: color.white
    },

    dropdown__arrow_leftMarged: {
        marginLeft: units(.75)
    },

    dropdown__arrow_rightMarged: {
        marginRight: units(.75)
    },

    dropdown__arrow_mono: {
        fill: color.black
    },

    dropdown__arrow_primary: {
        fill: color.primary
    },

    dropdown__arrow_secondary: {
        fill: color.secondary
    },

    dropdown_disabled: {
        opacity: .5,
        pointerEvents: 'none'
    },

    dropdown__container: {
        display: 'inline-block',
        boxSizing: 'border-box'
    },

    dropdown__container_relative: {
        position: 'relative'
    },

    dropdown__container_asAButton: {
        marginTop: 0,
        marginBottom: units(.5),
    },

    dropdown__container_noSpacing: {
        marginBottom: 0
    },

    dropdown__list: {
        margin: 0,
        padding: 0,
        listStyle: 'none'
    },

    dropdown__list__item: {

    }
}))(Dropdown);

const DropdownMenuWithStyles = withStyles(({ color, units, radius }) => ({
    dropdownMenu: {
        position: 'absolute',
        marginTop: units(.25),
        top: '100%',
        backgroundColor: color.white,
        boxShadow: `0 ${units(0.5)} ${units(0.75)} ${units(-0.45)} ${color.shadow}`,
        padding: units(0.5),
        borderRadius: radius,
        borderWidth: 1,
        borderStyle: 'solid',
        borderColor: color.blueGrayLight,
        zIndex: 50,
        opacity: 0.01,
        transform: 'translate(0, -10%)',
        transition: 'opacity 200ms ease-out, transform 300ms ease-out'
    },

    dropdownMenu_left: {
        left: 0
    },

    dropdownMenu_right: {
        right: 0
    },

    dropdownMenu_insideAButton: {
        marginRight: units(0.125),
        marginLeft: units(0.125)
    },

    //transitions
    dropdownMenu_entering: {
        opacity: 1,
        transform: 'translate(0, 0)',
        transition: 'opacity 200ms ease-out, transform 300ms ease-out'
    },

    dropdownMenu_exiting: {
        opacity: 0.01,
        transform: 'translate(0, -10%)',
        transition: 'opacity 200ms ease-out, transform 300ms ease-out'
    }

}), { flushBefore: true })(DropdownMenu);

export default DropDownWithStyles
