import React from 'react'
import PropTypes from 'prop-types'
import Loader from './Loader'
import Shimmer from './Shimmer'
import Icon from './Icon'
import { withStyles } from '../helpers/withStyles'
import { withStylesPropTypes } from 'react-with-styles'

const propTypes = {
    ...withStylesPropTypes,
    disabled: PropTypes.bool,
    loading: PropTypes.bool,
    icon: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
    inverse: PropTypes.bool,
    large: PropTypes.bool,
    mono: PropTypes.bool,
    primary: PropTypes.bool,
    secondary: PropTypes.bool,
    shimmer: PropTypes.bool,
    small: PropTypes.bool,
    shadowLevel: PropTypes.oneOf([0,1,2,3]),
    soft: PropTypes.bool,
    square: PropTypes.bool,
    transparent: PropTypes.bool,
    children: PropTypes.node
};

const defaultProps = {
    shadowLevel: 0
};

const FlatButton = (
    {
        styles,
        children,
        disabled,
        loading,
        icon,
        inverse,
        mono,
        soft,
        primary,
        secondary,
        shimmer,
        transparent,

        theme,
        css,

        href,
        large,
        noWrap,
        shadowLevel,
        small,
        ...props
    }
) => {
    return (
        <button
            {...props}
            {...css(
                styles.button,
                loading     && styles.button_loading,
                disabled    && styles.button_disabled,
                inverse     && styles.button_inverse,
                mono        && styles.button_mono,
                small       && styles.button_small,
                soft        && styles.button_soft,
                primary     && styles.button_primary,
                secondary   && styles.button_secondary,
                transparent && styles.button_transparent,
            )}
            disabled={disabled || loading}
            type="button"
        >
            {
                loading && shimmer &&

                <div{...css(styles.shimmer)}>
                    <Shimmer height="100%" width="100%" />
                </div>
            }
            <div
                {...css(
                    styles.button__content,
                    loading && styles.button__content_loading,
                )}
            >
                {
                    typeof icon === 'string'
                        ? <Icon name={icon} small={small} large={large} />
                        : icon
                }
            </div>
            { loading &&
            <div {...css(styles.button__loader)}>
                { !shimmer && <Loader small primary={inverse && soft} secondary={secondary} mono={mono} /> }
            </div>
            }
        </button>
    )
};

FlatButton.propTypes = propTypes;
FlatButton.defaultProps = defaultProps;

const FlatButtonWithStyles = withStyles(({ color, units, radius, font }) => ({
    button: {
        appearance: 'none',
        borderWidth: 0,
        background: color.secondary,
        height: units(2.5),
        width: units(2.5),
        padding: 0,
        borderRadius: '50%',
        color: color.white,
        fill: color.white,
        outline: 0,
        position: 'relative',
        margin: 0,
        fontWeight: 500,
        fontFamily: font.default,
        cursor: 'pointer',
        boxSizing: 'border-box',
        overflow: 'hidden',

        ':hover': {
            background: color.secondarySaturated
        },

        ':active': {
            background: color.secondaryDarker
        }
    },

    button_small: {
        width: units(1.75),
        height: units(1.75),
    },

    button_soft: {
        background: color.blueGrayLighter,
        color: color.primary,
        fill: color.primary,

        ':hover': {
            background: color.blueGrayLight
        },

        ':active': {
            background: color.blueGray
        }
    },

    button_loading: {
        opacity: 1,
        pointerEvents: 'none'
    },

    button_disabled: {
        opacity: .5,
        pointerEvents: 'none'
    },

    button_mono: {
        background: color.transparent,
        borderColor: color.black,
        borderWidth: 1,
        borderStyle: 'solid',
        color: color.black,
        fill: color.black,

        ':hover': {
            background: 'none'
        },

        ':active': {
            background: color.grayLighter
        }
    },

    button_primary: {
        background: color.primary,

        ':hover': {
            background: color.primarySaturated
        },

        ':active': {
            background: color.primaryDarker
        }
    },

    button_secondary: {
        background: color.transparent,
        borderColor: color.secondary,
        borderWidth: 1,
        borderStyle: 'solid',
        color: color.secondary,
        fill: color.secondary,

        ':hover': {
            color: color.secondarySaturated,
            fill: color.secondarySaturated,
            background: color.grayLightest
        },

        ':active': {
            borderColor: color.secondaryDarker,
            background: color.grayLighter
        }
    },

    button_inverse: {
        background: color.white,
        color: color.primary,
        fill: color.primary,
        borderWidth: 0,

        ':hover': {
            background: color.blueGrayLight
        },

        ':active': {
            background: color.blueGray
        }
    },

    button_transparent: {
        background: color.transparent,
        color: color.primary,
        fill: color.primary,

        ':hover': {
            background: color.shadowWeakest
        },

        ':active': {
            background: color.shadowWeaker
        }
    },

    button_square: {
        borderRadius: radius
    },

    button__content: {
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
        opacity: 1,
        transition: 'opacity .2s',
        whiteSpace: 'nowrap',
        userSelect: 'none'
    },

    button__content_loading: {
        opacity: 0,
    },

    button__loader: {
        position: 'absolute',
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center'
    },

    shimmer: {
        position: 'absolute',
        width: '100%',
        height: '100%',
        top: 0,
        right: 0,
        bottom: 0,
        left: 0
    }

}))(FlatButton);
export default FlatButtonWithStyles
