import React, { Component } from 'react'
import PropTypes from 'prop-types'
import InputOnly from './InputOnly'
import Spacing from './Spacing'
import { withScriptjs, withGoogleMap, GoogleMap, Marker } from 'react-google-maps'
import DrawingManager from 'react-google-maps/lib/components/drawing/DrawingManager'
import SearchBox from 'react-google-maps/lib/components/places/SearchBox'
import MarkerClusterer from 'react-google-maps/lib/components/addons/MarkerClusterer'
import theme from '../themes/CinnectaTheme'
import MapCircle from "./MapCircle";
import MapPolygon from "./MapPolygon";

const propTypes = {
    children: PropTypes.any,
    defaultZoom: PropTypes.number,
    defaultCenter: PropTypes.object,
    defaultClickableIcons: PropTypes.bool,
    editMode: PropTypes.bool,
    isCenterMarkerVisible: PropTypes.bool,
    places: PropTypes.bool,
    shapes: PropTypes.array,
    shapesToEnframe: PropTypes.array,
};

const defaultProps = {
    defaultZoom: 5,
    defaultCenter: { lat: -19.932231, lng: -43.9266687 },
    isCenterMarkerVisible: false,
    defaultOptions: {
        streetViewControl: false,
        scaleControl: false,
        mapTypeControl: false,
        panControl: false,
        zoomControl: false,
        rotateControl: false,
        fullscreenControl: false,
        clickableIcons: false,
        disableDefaultUI: true,
        styles: theme.map
    },

    defaultClickableIcons: false,
};

const shapeOptions = {
    fillColor: '#ee430e',
    strokeColor: '#ee430e',
    strokeWeight: 1,
    fillOpacity: .3,
    strokeOpacity: .5
};

class Map extends Component {
    constructor(props) {
        super(props);

        this.shapes = {};
        this.state = {
            google: null,
            pristine: true
        }
    }

    componentDidMount() {
        if(this.props.shapesToEnframe) {
            this.centerOnShapes(this.props.shapesToEnframe)
        }
    }

    componentDidUpdate(prevProps) {
        if(prevProps.shapesToEnframe !== this.props.shapesToEnframe) {
            this.centerOnShapes(this.props.shapesToEnframe)
        }
    }

    mapLoaded(ref) {
        if(ref && this.state.pristine) {
            this.map = ref;
            this.props.onMapLoaded && this.props.onMapLoaded()
            const google = ref.props.google;
            if(this.state.google !== google) {
                this.setState({ google, pristine: false }, () => this.centerOnShapes(this.props.shapes))
            }
        }
    }

    centerOnShapes(shapes) {
        const { google } = this.state

        if(shapes && shapes.length > 0 && google) {
            let bounds = new google.maps.LatLngBounds();
            shapes.map(shape => {
                if(shape.type === 'circle') {
                    const { lat, lng } = shape.center;
                    const latLng = new google.maps.LatLng(lat, lng);
                    const latOffset = shape.radius/111320;
                    const lngOffset = shape.radius/((40075000 * Math.cos(lat)/360));
                    const leftLatLng = new google.maps.LatLng(lat - latOffset, lng);
                    const rightLatLng = new google.maps.LatLng(lat + latOffset, lng);
                    const topLatLng = new google.maps.LatLng(lat, lng - lngOffset);
                    const bottomLatLng = new google.maps.LatLng(lat, lng + lngOffset);
                    bounds.extend(latLng)
                    bounds.extend(leftLatLng)
                    bounds.extend(rightLatLng)
                    bounds.extend(topLatLng)
                    bounds.extend(bottomLatLng)
                } else {
                    shape.path.map(node => {
                        const { lat, lng } = node;
                        const latLng = new google.maps.LatLng(lat, lng);
                        bounds.extend(latLng)
                        return true
                    })
                }

                return true
            });

            this.map.fitBounds(bounds)
        }
    }


    mapMoved() {
        this.props.onMapMoved && this.props.onMapMoved(this.map.getCenter())
        //console.log('mapMoved: ' + JSON.stringify(this.map.getCenter()))
    }

    zoomChanged() {
        this.props.onZoomChanged && this.props.onZoomChanged(this.map.getZoom())
        //console.log(this.map.getZoom())
    }

    placesChanged(places) {
        places && places[0] && this.map.fitBounds(places[0].geometry.viewport)
        this.props.onPlacesChanged && this.props.onPlacesChanged(places)
    }

    render() {
        const { children, defaultCenter, isCenterMarkerVisible, editMode, highlightShape, places, shapes, ...rest } = this.props;
        const { google } = this.state;

        return (
            //google &&
            <MapComponent
                loadingElement={<div style={{ height: `100%` }} />}
                containerElement={<div style={{ height: `100%` }} />}
                mapElement={<div style={{ height: `100%` }} />}
                googleMapURL="https://maps.googleapis.com/maps/api/js?key=AIzaSyBB6jb6anK2rjgXJDmXNfZzeFuiDOCqtu8&language=pt-BR&v=3.exp&libraries=geometry,drawing,places"
                defaultCenter={defaultCenter}
                mapMoved={() => this.mapMoved()}
                zoomChanged={() => this.zoomChanged()}
                mapLoaded={ref => this.mapLoaded(ref)}
                {...rest}
            >
                { isCenterMarkerVisible && <Marker position={defaultCenter} /> }
                {
                    editMode && google &&
                    <DrawingManager
                        defaultDrawingMode={google.maps.drawing.OverlayType.CIRCLE}
                        defaultOptions={{
                            drawingControl: true,
                            drawingControlOptions: {
                                position: google.maps.ControlPosition.TOP_RIGHT,
                                drawingModes: [
                                    google.maps.drawing.OverlayType.CIRCLE,
                                    google.maps.drawing.OverlayType.POLYGON,
                                ],
                            },
                            circleOptions: {
                                fillColor: `#68A7F0`,
                                strokeWeight: 0,
                                clickable: false,
                                editable: true,
                                zIndex: 1,
                            },
                            polygonOptions: {
                                fillColor: `#68A7F0`,
                                strokeWeight: 0,
                                clickable: false,
                                editable: true,
                                zIndex: 1,
                            },
                        }}
                    />
                }
                {
                    places && google &&
                    <SearchBox
                        ref={ref => this.searchBox = ref}
                        controlPosition={google.maps.ControlPosition.TOP_LEFT}
                        onPlacesChanged={() => this.searchBox && this.placesChanged(this.searchBox.getPlaces())}
                        onClose={() => {}}
                        showSearchBox={true}
                        onSelectStory={() => {}}
                    >
                        <div style={{ maxWidth: 540, width: '100%', opacity: .9}}>
                            <Spacing top={1} left={1}>
                                <InputOnly
                                    floating
                                    type="text"
                                    placeholder="Pesquise no mapa..."
                                    name="googlePlaces"
                                />
                            </Spacing>
                        </div>
                    </SearchBox>
                }
                <MarkerClusterer
                    averageCenter
                    enableRetinaIcons
                    gridSize={60}
                    maxZoom={13}
                    minimumClusterSize={1}
                >
                    {
                        shapes &&
                        shapes.map(shape => {
                            if(shape.type === 'circle') {
                                return (
                                    <MapCircle
                                        key={shape.id}
                                        //ref={ref => this.shapes[shape.id] = ref}
                                        center={shape.center}
                                        radius={shape.radius}
                                        editable={shape.editable}
                                        draggable={shape.draggable}
                                        options={{
                                            ...shapeOptions,
                                            ...shape.color && {fillColor: shape.color},
                                            ...highlightShape === shape.id && {fillColor: 'yellow'},
                                            ...shape.color && {strokeColor: shape.color},
                                        }}
                                    />
                                )
                            } else {
                                return (
                                    <MapPolygon
                                        key={shape.id}
                                        //ref={ref => this.shapes[shape.id] = ref}
                                        path={shape.path}
                                        options={{
                                            ...shapeOptions,
                                            ...shape.color && { fillColor: shape.color },
                                            ...shape.color && { strokeColor: shape.color },
                                        }}
                                    />
                                )
                            }
                        })
                    }
                    { children }
                </MarkerClusterer>
            </MapComponent>
        )
    }
}

Map.propTypes = propTypes;
Map.defaultProps = defaultProps;

const MapComponent = withScriptjs(withGoogleMap(({ defaultCenter, mapMoved, mapLoaded, zoomChanged, ...props}) => {
    return <GoogleMap
        defaultCenter={defaultCenter}
        onIdle={mapMoved}
        ref={mapLoaded}
        onZoomChanged={zoomChanged}
        google={window.google}
        {...props}
    >
        { props.children }
    </GoogleMap>
}));

export default Map

