import React, { Component, Fragment } from 'react'
import PropTypes from 'prop-types'
import _ from 'lodash'
import moment from 'moment'
import ComboButton from './ComboButton'
import Text from './Text'
import Anchor from './Anchor'
import Spacing from './Spacing'
import StatusIndicator from './StatusIndicator'
import Grid from './Grid'
import Cell from './Cell'
import { Responsive } from './Responsive'
import Visibility from './Visibility'
import CheckboxOnly from './CheckboxOnly'
import { withStyles } from '../helpers/withStyles'
import { withStylesPropTypes } from 'react-with-styles'

const propTypes = {
    ...withStylesPropTypes,
    item: PropTypes.object,
    clean: PropTypes.bool,
    checked: PropTypes.bool,
    onChange: PropTypes.func,
    onAction: PropTypes.func,
    children: PropTypes.any,
    divided: PropTypes.bool,
    loading: PropTypes.bool,
    selectable: PropTypes.bool
};

const defaultProps = {
    item: {}
};

class SummaryItem extends Component {
    constructor(props) {
        super(props);
        this.state = {
            hover: false
        }
    };

    render() {
        const {
            item,
            clean,
            checked,
            onChange,
            onAction,
            children,
            divided,
            loading,
            selectable,
            styles,
            css
        } = this.props;
        const { hover } = this.state;

        return (
            <li
                {...css(
                    styles.summaryItem,
                    divided && styles.summaryItem_divided,
                )}
                onMouseOver={() => this.setState({ hover: true })}
                onMouseOut={() => this.setState({ hover: false })}
            >
                <Grid resetSpacing>
                    <Visibility showIf={selectable} unmount>
                        <Responsive showFor="medium" unmount>
                            <div
                                {...css(
                                    styles.summaryItem__checkboxContainer,
                                    clean && styles.summaryItem__checkboxContainer_clean,
                                    clean && hover && styles.summaryItem__checkboxContainer_visible,
                                )}
                            >
                                <Cell shrink noSpacing={clean}>
                                    <CheckboxOnly checked={checked} onChange={() => onChange()}/>
                                </Cell>
                            </div>
                        </Responsive>
                    </Visibility>
                    {children && <Cell shrink>{children}</Cell>}
                    <Cell auto noSpacing>
                        <Grid>
                            <Cell auto maxWidth={24}>
                                <Text loading={loading} bold>
                                    <Anchor
                                        onClick={() => onAction(item.clickAction ? item.clickAction : item.mainAction)}
                                    >
                                        { item.title }
                                    </Anchor>
                                </Text>
                                <Text small shimmerWidth="100%" loading={loading}>{ item.description }</Text>
                                <Text small bold
                                      loading={loading}>{item.params && Array.isArray(item.params) && item.params.join(', ')}</Text>
                                { item.log &&
                                    <Spacing top={.5}>
                                        <Text muted small shimmerWidth="80%" loading={loading}>
                                            <Fragment>
                                                {_.capitalize(item.log.action)}
                                                <strong>
                                                    {
                                                        item.log.datetime &&
                                                        //moment(item.log.datetime).format('DD [de] MMMM [de] YYYY [às] hh:mm')
                                                        moment.utc(item.log.datetime).fromNow()
                                                    }
                                                </strong>
                                                {item.log.user && <span> por {
                                                    item.log.link ?
                                                        <strong>
                                                            <a href={item.log.link}>
                                                                {item.log.user}
                                                            </a>
                                                        </strong>
                                                        : <strong>{item.log.user}</strong>
                                                } </span>}
                                            </Fragment>
                                        </Text>
                                    </Spacing>
                                }
                            </Cell>
                            <Cell noSpacing size={12} largeSize={2}>
                                {
                                    item.status &&

                                    <Spacing
                                        top={1}
                                        topLargeAndAbove={0.1}
                                        left={1}
                                        leftLargeAndAbove={0.25}>
                                        <StatusIndicator
                                            loading={loading}
                                            text={item.status.label}
                                            color={item.status.color}
                                        />
                                    </Spacing>
                                }
                            </Cell>
                            <Cell size={12} autoOnMedium center>
                                {
                                    item.report && Array.isArray(item.report) && item.report.length > 0 &&
                                    <Spacing top={1.25} topLargeAndAbove={0}>
                                        <div {...css(styles.summaryItem__info)}>
                                            {
                                                item.report.map(data => {
                                                    if (data) {
                                                        return (
                                                            <div {...css(styles.summaryItem__info__item)} key={data.label}>
                                                                <strong {...css(styles.summaryItem__info__value)}>{data.value}</strong>
                                                                <span {...css(styles.summaryItem__info__label)}>{data.label}</span>
                                                            </div>
                                                        )
                                                    } else {
                                                        return false
                                                    }
                                                })}
                                        </div>
                                    </Spacing>
                                }
                            </Cell>
                        </Grid>
                    </Cell>
                    <Cell size={12} shrinkOnMedium>
                        <div {...css(
                            styles.summaryItem__actions,
                            clean && styles.summaryItem__actions_clean,
                            clean && hover && styles.summaryItem__actions_visible
                        )}>
                            { item.customButtom }
                            <ComboButton
                                soft
                                loading={loading}
                                shimmer={loading}
                                mainOption={item.actions && item.actions.filter(action => item.mainAction === action.value)[0]}
                                options={item.actions ? item.actions.filter(action => item.mainAction !== action.value) : []}
                                onClick={option => onAction && onAction(option)}
                                closeOnSelect
                                noSpacing
                            />
                        </div>
                    </Cell>
                </Grid>
            </li>
        )
    }
}

SummaryItem.propTypes = propTypes;
SummaryItem.defaultProps = defaultProps;

const SummaryItemWithStyles = withStyles(({ color, units, breakpoint, gutter, fontSize, font, weight }) => ({
    summaryItem: {
        padding: 0,
        paddingTop: units(1.25),
        paddingBottom: units(1.25),
        position: 'relative',
        display: 'flex',
        fontFamily: font.default
    },

    summaryItem_divided: {
        borderBottomWidth: 1,
        borderBottomStyle: 'solid',
        borderBottomColor: color.blueGrayLight,
    },

    summaryItem__checkboxContainer: {
        flexShrink: 1,
        flexGrow: 0
    },

    summaryItem__checkboxContainer_clean: {
        position: 'absolute',
        paddingLeft: units(1),
        transform: 'translate(calc(-100% + .5rem), 0)',
        visibility: 'visible',
        height: '100%',
        transition: 'visibility .25s ,opacity .25s',

        [breakpoint.large]: {
            visibility: 'hidden',
            opacity: 0.01
        }
    },

    summaryItem__checkboxContainer_visible: {
        [breakpoint.large]: {
            visibility: 'visible',
            opacity: 1
        }
    },

    summaryItem__actions: {
        marginTop: units(1),

        [breakpoint.mediumAndAbove]: {
            marginTop: 0,
        }

    },

    summaryItem__actions_clean: {
        visibility: 'visible',
        transition: 'visibility .25s ,opacity .25s',

        [breakpoint.large]: {
            visibility: 'hidden',
            opacity: 0.01,
        }
    },

    summaryItem__actions_visible: {
        [breakpoint.large]: {
            visibility: 'visible',
            opacity: 1,
        }
    },

    summaryItem__content: {
        flex: 1
    },

    summaryItem__info: {
        display: 'flex'
    },

    summaryItem__info__item: {
        width: units(5),
        padding: units(.25),
        textAlign: 'left',

        [breakpoint.large]: {
            textAlign: 'center'
        }
    },

    summaryItem__info__value: {
        fontSize: fontSize.small.large,
        fontWeight: weight.bold,
        color: color.primary,
        lineHeight: 1,
        display: 'block',

        [breakpoint.large]: {
            fontSize: fontSize.large.large,
        }
    },

    summaryItem__info__label: {
        fontSize: fontSize.small.small,
        color: color.grayDarker,

        [breakpoint.large]: {
            fontSize: fontSize.large.small,
        }
    },

}))(SummaryItem);

export default SummaryItemWithStyles
