import React, { Component, Fragment } from 'react'
import PropTypes from 'prop-types'
import _ from 'lodash'
import moment from 'moment'
import Dropdown from './Dropdown'
import Grid from './Grid'
import Cell from './Cell'
import { Responsive } from './Responsive'
import Visibility from './Visibility'
import CheckboxOnly from './CheckboxOnly'
import InputOnly from './InputOnly'
import ThrottledTextInput from './ThrottledTextInput'
import Icon from './Icon'
import Text from './Text'
import Spacing from './Spacing'
import SummaryItem from './SummaryItem'
import { withStyles } from '../helpers/withStyles'

const propTypes = {
    data: PropTypes.array.isRequired,
    loading: PropTypes.bool,
    dateFormat: PropTypes.string,
    divided: PropTypes.bool,
    searchPlaceholder: PropTypes.string,
    maxResults: PropTypes.number,
    noResultComponent: PropTypes.object,
    customActions: PropTypes.any,
    sortOptions: PropTypes.arrayOf(PropTypes.objectOf(PropTypes.string)),
    filterFor: PropTypes.arrayOf(PropTypes.oneOf(['title', 'description'])),
    clean: PropTypes.bool,
    onItemAction: PropTypes.func,
    paginate: PropTypes.bool,
    icons: PropTypes.arrayOf(PropTypes.string),
    withFilter: PropTypes.bool,
    sortable: PropTypes.bool,
    selectableItems: PropTypes.bool,
};

const defaultProps = {
    data: [],
    dateFormat: 'YYYY-MM-DD hh:mm:ss',
    maxResults: 0,
    noResultComponent: <div>Nenhum resultado...</div>,
    searchPlaceholder: 'buscar...',
    sortOptions: [
        { label: 'Data de criação', value: 'createdAt' },
        { label: 'Última atualização', value: 'updatedAt' },
        { label: 'Título', value: 'title' },
    ],
    filterFor: ['title', 'description']
};

class SummaryList extends Component {
    constructor(props) {
        super(props);
        this.state = {
            checked: [],
            sort: props.sortOptions ? props.sortOptions[0] : null,
            filterString: ''
        }
    }

    filterList(data) {
        const { filterFor, withFilter } = this.props;
        const { filterString } = this.state;

        if(
            withFilter &&
            Array.isArray(filterFor) &&
            filterFor.length > 0 &&
            filterString.length > 2
        ) {
            return data.filter(item => {
                return filterFor.some(filter => {
                    try {
                        return item[filter].toLowerCase().search(filterString.toLowerCase()) !== -1
                    } catch (e) {
                        return false
                    }
                })
            })
        }

        return data
    }

    sortList(data) {
        const { sort } = this.state;
        if(sort.value && data && Array.isArray(data) && data.length > 0 && data[0][sort.value] != null) {
            const isTime = moment(data[0][sort.value], this.props.dateFormat).isValid();

            return _.orderBy(data, item => {
                return isTime ? moment(item[sort.value], this.props.dateFormat).valueOf() : item[sort.value]
            }, isTime ? ['desc'] : ['asc'])
        }

        return data
    }

    toggleItem(id) {
        let checked = [...this.state.checked];

        if(checked.includes(id)) {
            checked.splice(id, 1)
        } else {
            checked.push(id)
        }

        this.setState({ checked })
    }

    render() {
        const {
            data,
            loading,
            divided,
            searchPlaceholder,
            noResultComponent,
            maxResults,
            customActions,
            sortOptions,
            clean,
            paginate,
            onItemAction,
            icons,
            withFilter,
            selectableItems,
            sortable,
            styles,
            css,
        } = this.props;
        const { checked } = this.state;

        const listPlaceholder = loading ? (
            <Fragment>
                { [...Array(maxResults ? maxResults : 10)].map((e, i) => <SummaryItem divided={divided} key={i} loading />) }
            </Fragment>
        ) : (
            <div>{ noResultComponent }</div>
        );
        return (
            <div
                {...css(
                    styles.summaryList,
                    clean && styles.summaryList_clean,
                    checked.length > 0 && styles.summaryList_edit
                )}
            >
                {(withFilter || sortable || selectableItems) &&
                    <div {...css(styles.summaryList__header)}>
                        <Grid resetSpacing>
                            {
                                selectableItems &&

                                <Cell shrink>
                                    <CheckboxOnly />
                                </Cell>
                            }
                            {
                                withFilter &&

                                <Responsive showFor="large">
                                    <Cell auto>
                                        <div {...css(styles.summaryList__header__filter)}>
                                            <ThrottledTextInput
                                                debouncedOnChange={value => this.setState({filterString: value})}
                                                debounceTimeoutMs={500}
                                            >
                                                <InputOnly
                                                    name="filterInput"
                                                    placeholder={searchPlaceholder}
                                                    shimmer={loading}
                                                    prefix={<Icon name="search" />}
                                                    removeMargins
                                                />
                                            </ThrottledTextInput>
                                        </div>
                                    </Cell>
                                </Responsive>
                            }
                            {
                                customActions &&

                                <Responsive showFor="large">
                                    <Cell shrink>
                                        {customActions}
                                    </Cell>
                                </Responsive>

                            }
                            {
                                sortable &&

                                <Cell shrink>
                                    <span>
                                        <Text small inline loading={loading}>Ordernar por:</Text>
                                        <Spacing left={.1} inline>
                                            <Dropdown
                                                button
                                                label={this.state.sort && this.state.sort.label}
                                                options={sortOptions}
                                                onSelectOption={value => this.setState({ sort: value })}
                                                shimmer={loading}
                                                loading={loading}
                                                closeOnSelect
                                                inverse
                                                compact
                                                noSpacing
                                            />
                                        </Spacing>
                                    </span>
                                </Cell>
                            }
                        </Grid>
                    </div>
                }
                <ul {...css(styles.summaryList__list)}>
                    { data.length < 1 ? listPlaceholder : (
                        this.sortList(this.filterList(data)).map(item => {
                        if(item) {
                            return (
                                <SummaryItem
                                    key={item.id}
                                    item={loading ? {} : item}
                                    clean={clean}
                                    divided={divided}
                                    checked={checked.includes(item.id)}
                                    onAction={action => onItemAction(action, item.id)}
                                    onChange={() => this.toggleItem(item.id)}
                                    loading={loading}
                                    selectable={selectableItems}
                                >
                                    { icons ? (
                                        <Fragment>
                                            {item.icons.map(
                                                icon => <i className={`icon-${icon}`} key={icon} />
                                            )}
                                        </Fragment>
                                    ) : null}
                                </SummaryItem>
                            )
                        } else {
                            return false
                        }
                    }))
                    }
                </ul>
                <Visibility unmount showIf={paginate}>
                    paginação
                </Visibility>
            </div>
        )
    }
}

SummaryList.propTypes = propTypes;
SummaryList.defaultProps = defaultProps;

const SummaryListWithStyles = withStyles(({ units, color, breakpoint }) => ({
    summaryList: {

    },

    summaryList_clean: {
        [breakpoint.mediumAndAbove]: {
            //paddingLeft: units(2),
            //paddingRight: units(2),
        }
    },

    summaryList_edit: {

    },

    summaryList__header: {
        paddingTop: 0,
        paddingBottom: units(1.5),
        borderBottomWidth: 1,
        borderBottomStyle: 'solid',
        borderBottomColor: color.blueGrayLight,
    },

    summaryList__header__filter: {
        maxWidth: units(24)
    },

    summaryList__list: {
        margin: 0,
        padding: 0,
        listStyle: 'none'
    }

}))(SummaryList);

export default SummaryListWithStyles
